@@ -17,7 +17,6 @@
  */
 
 #include "kerncompat.h"
-#include "androidcompat.h"
 
 #include <sys/ioctl.h>
 #include <sys/wait.h>
@@ -42,7 +41,8 @@
 #include "common/utils.h"
 #include "kernel-shared/volumes.h"
 #include "kernel-shared/disk-io.h"
-
+#include "common/open-utils.h"
+#include "common/units.h"
 #include "cmds/commands.h"
 #include "common/help.h"
 
